function mtl2vtu(FILE,NOD,ELE,SOL)
% mtl2vtu(F2WRITE,NOD,ELE,SOL)
% Writes file F2WRITE in vtu format.
% NOD can be a structure or a matrix with nodal coordinates
%    a) if ismatrix(NOD), NOD(i,j): j-coordinate of node i (j=1,2(,3), i=1,2,...,nnod)
%    b) is isstruct(NOD), NOD must have the field Coordinates. 
%                         NOD.Coordinates(i,j): j-coordinate of node i  
% ELE can be a structure or a matrix with element connectivities
%    a) if ismatrix(ELE), ELE(i,j): j-th node of element i (j=1,2,3, i=1,2,...,nele)
%    b) if isstruct(ELE), ELE must have the fields Connectivity and Type
%                         ELE.Connectivity(i,j): j-th node of element i
%                         ELE.Type: finite element type   
% SOL can be a vector, a structure or empty
%    a) if isvector(SOL), Displacement = SOL
%    b) if isstruct(SOL), SOL(i) (i=1,...,nbr of quantities to display)
%       is a structure with 4 fields:
%                   SOL(i).Name
%                   SOL(i).Type (PointData or CellData)
%                   SOL(i).NumberOfComponents
%                   SOL(i).Value
%    c) if isempty(SOL), only the mesh is displayed.
%
% mtl2vtu(F2OPEN,NOD,ELE)
% Opens file F2OPEN in vtu format with nodes and elements data.
% F2OPEN remains un-closed and cannot be viewed.
%
% mtl2vtu(F2OPEN,CLOSEDF,SOL)
% Adds SOL to F2OPEN and closes it. The resulting file is CLOSEDF.

clos = 0;
ope = 0;
if nargin == 3
    if ischar(NOD)
        % FILE = mtl2vtu(F2OPEN,CLOSEDF,SOL)
        CLOSEDF = NOD;
        SOL = ELE;
        clos = 1;
    else
        ope = 1;
    end
end

if clos
    copyfile(FILE,CLOSEDF);
    FID = fopen(CLOSEDF,'a');
else
    FID = fopen(FILE,'w');
    fprintf(FID, '<?xml version="1.0"?>\n');
    fprintf(FID, '<VTKFile type="UnstructuredGrid" version="0.1" byte_order="LittleEndian">\n');
    fprintf(FID, '    <UnstructuredGrid>\n');
    
    if isstruct(NOD)
        NOD = NOD.Coordinates;
    end
    
    [nnod,dim] = size(NOD);
    if dim==2
        NOD(1,3) = 0;
    end
    if isstruct(ELE)
        ETYPE = ELE.Type;
        ELE = ELE.Connectivity;
        [nele,nnodperele] = size(ELE);
    else
        [nele,nnodperele] = size(ELE);
        switch(nnodperele)
            case {2}
                ETYPE = 'BAR2';
            case {3}
                ETYPE = 'TRI3';
                warning('Element is assumed to be TRI3');
            case {4}
                ETYPE = 'QUAD4';
                warning('Element is assumed to be QUAD4');
            case {8}
                ETYPE = 'HEXA8';
                warning('Element is assumed to be HEXA8');
            case {9}
                ETYPE = 'QUAD9';
                warning('Element is assumed to be QUAD9');
            case {16}
                ETYPE = 'QUAD16';
                warning('Element is assumed to be QUAD16');
        end
    end
    
    switch(ETYPE)
        case {'BAR2'}
            etype = 3; %VTK_LINE
        case {'TRI3'}
            etype = 5; %VTK_TRIANGLE
        case {'QUAD4'}
            etype = 9; % VTK_QUAD
        case {'QUAD8'}
            etype = 23; %VTK_QUADRATIC_QUAD
        case {'QUAD9'}
            etype = 28; %VTK_BIQUADRATIC_QUAD
        case {'QUAD16'}
            % etype = 36; %VTK_BICUBIC_QUAD
            % No funciona. Paraview 3.12.0 no acepta cuadrngulos bi-cbicos.
            % Para visualizacin, cada cuadrngulo bi-cbico es dividido en
            % 9 cuadrngulos bi-lineales.
            etype = 9; % VTK_QUAD
            C = ELE;
            ELE = [C(:,[1 5 13 12])
                C(:,[5 6 14 13])
                C(:,[6 2 7 14])
                C(:,[12 13 16 11])
                C(:,[13 14 15 16])
                C(:,[14 7 8 15])
                C(:,[11 16 10 4])
                C(:,[16 15 9 10])
                C(:,[15 8 3 9])];
            nele = 9*nele;
            nnodperele = 4;
        case {'TETRA4'}
            etype = 10; %VTK_TETRA
        case {'HEXA8','HEXA8RI','HEXA8MITC4','VOLHEXA8'}
            etype = 12; %VTK_HEXAHEDRON
        case {'HEXA20'}
            etype = 25; %VTK_QUADRATIC_HEXAHEDRON
        case {'HEXA27'}
            etype = 29; %VTK_TRIQUADRATIC_HEXAHEDRON
            ELE = ELE(:,[1:20 24 26 23 25 21 22 27]);
%             nnodperele = 20;
%             ELE = ELE(:,1:20);
        otherwise
            error('Unknown element')
    end
    
    NOD = NOD';
    ELE = ELE';
    
    fprintf(FID, '        <Piece NumberOfPoints="%d" NumberOfCells="%d">\n',nnod,nele);
    
    fprintf(FID, '            <Points>\n');
    fprintf(FID, '                <DataArray type="Float32" NumberOfComponents="3" format="ascii">\n');
    N = num2str(NOD(:)',' %f');
    fprintf(FID, ['                    ' N '\n']);
    fprintf(FID, '                </DataArray>\n');
    fprintf(FID, '            </Points>\n');
    
    fprintf(FID, '            <Cells>\n');
    fprintf(FID, '                <DataArray type="Int32" Name="connectivity" format="ascii">\n');
    N = num2str(ELE(:)'-1,' %d');
    fprintf(FID, ['                    ' N '\n']);
    fprintf(FID, '                </DataArray>\n');
    fprintf(FID, '                <DataArray type="Int32" Name="offsets" format="ascii">\n');
    N = num2str(nnodperele:nnodperele:nnodperele*nele,' %d');
    fprintf(FID, ['                    ' N '\n']);
    fprintf(FID, '                </DataArray>\n');
    fprintf(FID, '                <DataArray type="UInt8" Name="types" format="ascii">\n');
    N = num2str(etype*ones(1,nele),' %d');
    fprintf(FID, ['                    ' N '\n']);
    fprintf(FID, '                </DataArray>\n');
    fprintf(FID, '            </Cells>\n');
end

if ope == 0
    if ~isempty(SOL)
        if isstruct(SOL)
            nsol = length(SOL);
            pdata = [];
            cdata = [];
            for i=1:nsol
                if strcmp(SOL(i).Type,'PointData')
                    pdata = [pdata i];
                else
                    cdata = [cdata i];
                end
            end
        elseif exist('nnod','var')
            [i,j] = size(SOL);
            if i == nnod || i==dim*nnod
                d = 'PointData';
                pdata = 1;
                cdata = [];
                if nnod==nele
                    warning('SOL is assumed to be PointData')
                end
                if j>1
                    n = j;
                else
                    n = i/nnod;
                end
            elseif i==nele || i==dim*nele
                d = 'CellData';
                cdata = 1;
                pdata = [];
                if j>1
                    n = j;
                else
                    n = i/nele;
                end
            else
                error('Wrong number of components in SOL')
            end
            if j>1
                l = i*j;
                V = zeros(l,1);
                for i = 1:j
                    V(i:j:end) = SOL(:,i);
                end
            else
                V = SOL;
            end
            SOL = struct('Name','Results','Type',d,'NumberOfComponents',n,'Value',V);
        else
            error('SOL must be a structure')
        end
        
        if ~isempty(pdata)
            fprintf(FID, '            <PointData>\n');
            for i = pdata
                sol = SOL(i);
                fprintf(FID, ['                <DataArray type="Float32" Name="' sol.Name '" NumberOfComponents="' num2str(sol.NumberOfComponents) '" format="ascii">\n']);
                v = sol.Value;
                [n,d] = size(v);
                if d > 1
                    if sol.NumberOfComponents ~= d
                        error('Wrong number of components')
                    end
                    V = zeros(1,d*n);
                    for j=1:d
                        V(j:d:end) = v(:,j)';
                    end
                else
                    V = v';
                end
                N = num2str(V,' %f');
                fprintf(FID, ['                    ' N '\n']);
                fprintf(FID, '                </DataArray>\n');
            end
            fprintf(FID, '            </PointData>\n');
        end
        if ~isempty(cdata)
            fprintf(FID, '            <CellData>\n');
            for i = cdata
                sol = SOL(i);
                fprintf(FID, ['                <DataArray type="Float32" Name="' sol.Name '" NumberOfComponents="' num2str(sol.NumberOfComponents) '" format="ascii">\n']);
                v = sol.Value;
                [n,d] = size(v);
                if d > 1
                    if sol.NumberOfComponents ~= d
                        error('Wrong number of components')
                    end
                    V = zeros(1,d*n);
                    for j=1:d
                        V(j:d:end) = v(:,j)';
                    end
                else
                    V = v';
                end
                N = num2str(V,' %f');
                fprintf(FID, ['                    ' N '\n']);
                fprintf(FID, '                </DataArray>\n');
            end
            fprintf(FID, '            </CellData>\n');
        end
    end
    fprintf(FID, '        </Piece>\n');
    fprintf(FID, '    </UnstructuredGrid>\n');
    fprintf(FID, '</VTKFile>\n');
end
fclose(FID);
%toc